'''
Functiile de mai jos au fost folosite pentru a calcula metrica si scorul pentru fiecare subtask individual

Fiecare primeste un parametru ("submission") care este un dataframe cu 3 coloane:
    - ID = id-ul din setul de date de test
    - correct_values = valorile din ground truth
    - output_values = valorile trimise de voi

Pentru fiecare subtask, dataframe-ul cu care se apeleaza functia va contine doar liniile corespunzatoare
subtaskului respectiv

Functiile returneaza:
    - partial_score = scorul pe o parte din setul de date de test
    - partial_metric = metrica pe o parte din setul de date de test
    - complete_score = scorul pe toate datele de test
    - complete_metric = metrica pe toate datele de test
'''

from sklearn import metrics, model_selection
import pandas as pd
import math

def subtask1(submission):
    output = submission['output_values'].iloc[0]
    correct = submission['correct_values'].iloc[0]

    if output == correct:
        return 3, 1, 3, 1
    else:
        return 0, 0, 0, 0

def subtask2(submission):
    output = submission['output_values'].iloc[0]
    correct = submission['correct_values'].iloc[0]

    if output == correct:
        return 4, 1, 4, 1
    else:
        return 0, 0, 0, 0

def subtask3(submission):
    output = float(submission['output_values'].iloc[0])
    correct = float(submission['correct_values'].iloc[0])

    if abs(output - correct) <= 0.001:
        return 6, 1, 6, 1
    else:
        return 0, 0, 0, 0

def subtask4(submission):
    output = submission['output_values'].iloc[0]
    correct = submission['correct_values'].iloc[0]

    if output == correct:
        return 7, 1, 7, 1
    else:
        return 0, 0, 0, 0

def subtask5(submission):
    submission['output_values'] = pd.to_numeric(submission['output_values'], errors='coerce').fillna(0).astype(int)

    public = submission[:1500]
    complete = submission

    def compute_score(metric):
        if metric < 1.6:
            return 60
        else:
            score = int(60 - 5 * math.sqrt((metric - 1.6) * 2.4))
            return max(0, score)

    # Public metric and score
    public_metric = metrics.mean_absolute_error(
        list(map(int, public['correct_values'])),
        list(map(int, public['output_values']))
    )
    public_score = compute_score(public_metric)

    # Complete metric and score
    complete_metric = metrics.mean_absolute_error(
        list(map(int, complete['correct_values'])),
        list(map(int, complete['output_values']))
    )
    complete_score = compute_score(complete_metric)

    return int(public_score), round(public_metric, 4), int(complete_score), round(complete_metric, 4)

def subtask6(submission):
    submission['output_values'] = pd.to_numeric(submission['output_values'], errors='coerce').fillna(0).astype(int)

    public = submission[:350]
    complete = submission

    def compute_score(metric):
        if metric < 1.2:
            return 20
        elif metric < 1.5:
            return 19
        else:
            score = int(19 - 3 * math.sqrt(metric - 1.5))
            return max(0, score)

    # Public metric and score
    public_metric = metrics.mean_absolute_error(
        list(map(int, public['correct_values'])),
        list(map(int, public['output_values']))
    )
    public_score = compute_score(public_metric)

    # Complete metric and score
    complete_metric = metrics.mean_absolute_error(
        list(map(int, complete['correct_values'])),
        list(map(int, complete['output_values']))
    )
    complete_score = compute_score(complete_metric)

    return int(public_score), round(public_metric, 4), int(complete_score), round(complete_metric, 4)