import pandas as pd
from sklearn.linear_model import LinearRegression
from sklearn.model_selection import cross_val_score

train_data = pd.read_csv('train_data.csv', index_col = 'User_ID')
y_train = train_data.pop('Calories')
test_data = pd.read_csv('test_data.csv')

# subtasks
subtask1 = len(train_data)
subtask2 = len(train_data.loc[train_data['Gender'] == 'male'])
subtask3 = train_data['Duration'].mean()
subtask4 = len(train_data.loc[train_data['Age'] >= 75])

test_data1 = test_data.loc[test_data['Subtask'] == 5]
test_data1 = test_data1.set_index('User_ID')

test_data2 = test_data.loc[test_data['Subtask'] == 6]
test_data2 = test_data2.set_index('User_ID')
test_data2.loc[:, 'Gender'] = 'male'

def encode_gender (dataframe):
    dataframe['Gender'] = dataframe['Gender'].map(lambda x : 0 if x == 'male' or x == 0 else 1)

def clean_data(dataframe):
    dropped = []
    dropped += ['Body_Temp']
    dropped += ['Height']
    dropped += ['Subtask']
    for column in dropped:
        if column in dataframe.columns:
            dataframe.pop(column)

for dataframe in [train_data, test_data1, test_data2]:
    encode_gender(dataframe)
    clean_data(dataframe)

columns = list(train_data.columns)
def extra_features(dataframe):
    feature_count = 0
    for col1 in columns:
        for col2 in columns:
            dataframe["feature" + str(feature_count)] = dataframe[col1] * dataframe[col2]
            feature_count += 1

for dataframe in [train_data, test_data1, test_data2]:
    extra_features(dataframe)

def train_model_gendered(gender, train_data, y_train):
    train_data_gendered = train_data.loc[train_data['Gender'] == gender]
    y_train_gendered = y_train.loc[train_data_gendered.index]

    model = LinearRegression()
    model.fit(train_data_gendered, y_train_gendered)

    # uncomment to test your model:
    # scores = cross_val_score(model, train_data_gendered, y_train_gendered, scoring = 'neg_mean_absolute_error', cv = 10)
    # print(-scores.mean())

    return model

def predict_gendered(gender, test_data, model):
    test_data_gendered = test_data.loc[test_data['Gender'] == gender]

    predictions = model.predict(test_data_gendered)
    return predictions

model_male = train_model_gendered(0, train_data, y_train)
model_female = train_model_gendered(1, train_data, y_train)

preds1_male = predict_gendered(0, test_data1, model_male)
preds1_female = predict_gendered(1, test_data1, model_female)
test_data1['answer'] = 0
test_data1.loc[test_data1['Gender'] == 0, 'answer'] = preds1_male
test_data1.loc[test_data1['Gender'] == 1, 'answer'] = preds1_female

preds2 = predict_gendered(0, test_data2, model_male)
test_data2['answer'] = preds2

sample = pd.read_csv('sample_output.csv')
sample.loc[sample['subtaskID'] == 1, 'answer'] = subtask1
sample.loc[sample['subtaskID'] == 2, 'answer'] = subtask2
sample.loc[sample['subtaskID'] == 3, 'answer'] = subtask3
sample.loc[sample['subtaskID'] == 4, 'answer'] = subtask4
sample.loc[sample['subtaskID'] == 5, 'answer'] = test_data1['answer'].set_axis(range(4, 2004))
sample.loc[sample['subtaskID'] == 6, 'answer'] = test_data2['answer'].set_axis(range(2004, 2504))
sample.to_csv('output.csv')
